"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toReadonlyArray = exports.intercalate = exports.reduceE = exports.reduceRight = exports.foldMap = exports.reduce = void 0;
var function_1 = require("./function");
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * `reduce` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var reduce = function (F, G) { return function (b, f) {
    return F.reduce(b, function (b, ga) { return (0, function_1.pipe)(ga, G.reduce(b, f)); });
}; };
exports.reduce = reduce;
/**
 * `foldMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var foldMap = function (F, G) {
    return function (M) { return function (f) { return F.foldMap(M)(G.foldMap(M)(f)); }; };
};
exports.foldMap = foldMap;
/**
 * `reduceRight` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
var reduceRight = function (F, G) {
    return function (b, f) { return F.reduceRight(b, function (ga, b) { return G.reduceRight(b, f)(ga); }); };
};
exports.reduceRight = reduceRight;
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * Similar to 'reduce', but the result is encapsulated in a monad.
 *
 * Note: this function is not generally stack-safe, e.g., for monads which build up thunks a la `IO`.
 *
 * @example
 * import { reduceE } from 'fp-ts/Foldable'
 * import { Chain, some } from 'fp-ts/Option'
 * import { tree, Foldable } from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const t = tree(1, [tree(2, []), tree(3, []), tree(4, [])])
 * assert.deepStrictEqual(pipe(t, reduceE(Foldable)(Chain)(some(0), (b, a) => (a > 2 ? some(b + a) : some(b)))), some(7))
 *
 * @since 3.0.0
 */
function reduceE(F) {
    return function (M) { return function (mb, f) {
        return F.reduce(mb, function (mb, a) {
            return (0, function_1.pipe)(mb, M.chain(function (b) { return f(b, a); }));
        });
    }; };
}
exports.reduceE = reduceE;
/**
 * Fold a data structure, accumulating values in some `Monoid`, combining adjacent elements using the specified separator.
 *
 * @example
 * import { intercalate } from 'fp-ts/Foldable'
 * import { Monoid } from 'fp-ts/string'
 * import * as T from 'fp-ts/Tree'
 * import { pipe } from 'fp-ts/function'
 *
 * const t = T.tree('a', [T.tree('b', []), T.tree('c', []), T.tree('d', [])])
 * assert.strictEqual(pipe(t, intercalate(T.Foldable)(Monoid)('|')), 'a|b|c|d')
 *
 * @since 3.0.0
 */
function intercalate(F) {
    return function (M) { return function (sep) { return function (fm) {
        var go = function (_a, m) {
            var init = _a[0], acc = _a[1];
            return init ? [false, m] : [false, (0, function_1.pipe)(acc, M.concat(sep), M.concat(m))];
        };
        return (0, function_1.pipe)(fm, F.reduce([true, M.empty], go))[1];
    }; }; };
}
exports.intercalate = intercalate;
/**
 * Transforms a `Foldable` into a read-only array.
 *
 * @example
 * import { toReadonlyArray } from 'fp-ts/Foldable'
 * import { Foldable, tree } from 'fp-ts/Tree'
 *
 * const t = tree(1, [tree(2, []), tree(3, []), tree(4, [])])
 * assert.deepStrictEqual(toReadonlyArray(Foldable)(t), [1, 2, 3, 4])
 *
 * @since 3.0.0
 */
function toReadonlyArray(F) {
    return function (fa) {
        return (0, function_1.pipe)(fa, F.reduce([], function (acc, a) {
            acc.push(a);
            return acc;
        }));
    };
}
exports.toReadonlyArray = toReadonlyArray;
