"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fanOut = exports.split = void 0;
var function_1 = require("./function");
/**
 * Compose a value acting on a tuple from two values, each acting on one of the components of the tuple.
 *
 * Specializing `split` to function application would look like this:
 *
 * ```purescript
 * split :: forall a b c d. (a -> b) -> (c -> d) -> (Tuple a c) -> (Tuple b d)
 * ```
 *
 * We take two functions, `f` and `g`, and we transform them into a single function which takes a tuple and maps `f`
 * over the first element and `g` over the second.  Just like `bi-map` would do for the `bi-functor` instance of tuple.
 *
 * @since 3.0.0
 */
var split = function (S, C) { return function (pab, pcd) {
    return (0, function_1.pipe)(S.first(pab), C.compose(S.second(pcd)));
}; };
exports.split = split;
/**
 * Compose a value which introduces a tuple from two values, each introducing one side of the tuple.
 *
 * This combinator is useful when assembling values from smaller components, because it provides a way to support two
 * different types of output.
 *
 * Specializing `fanOut` to function application would look like this:
 *
 * ```purescript
 * fanOut :: forall a b c. (a -> b) -> (a -> c) -> (a -> (Tuple b c))
 * ```
 *
 * We take two functions, `f` and `g`, with the same parameter type and we transform them into a single function which
 * takes one parameter and returns a tuple of the results of running `f` and `g` on the parameter, respectively.  This
 * allows us to run two parallel computations on the same input and return both results in a tuple.
 *
 * @since 3.0.0
 */
var fanOut = function (S, C) {
    var splitSC = (0, exports.split)(S, C);
    return function (pab, pac) {
        return (0, function_1.pipe)(C.id(), S.promap(function_1.identity, function (a) { return [a, a]; }), C.compose(splitSC(pab, pac)));
    };
};
exports.fanOut = fanOut;
