import { identity, pipe } from './function';
import * as _ from './internal';
/**
 * Compose a value acting on a sum from two values, each acting on one of
 * the components of the sum.
 *
 * Specializing `split` to function application would look like this:
 *
 * ```purescript
 * split :: forall a b c d. (a -> b) -> (c -> d) -> (Either a c) -> (Either b d)
 * ```
 *
 * We take two functions, `f` and `g`, and we transform them into a single function which
 * takes an `Either`and maps `f` over the left side and `g` over the right side.  Just like
 * `bimap` would do for the `Bifunctor` instance of `Either`.
 *
 * @since 3.0.0
 */
export const split = (P, C) => (pab, pcd) => pipe(P.left(pab), C.compose(P.right(pcd)));
/**
 * Compose a value which eliminates a sum from two values, each eliminating
 * one side of the sum.
 *
 * This combinator is useful when assembling values from smaller components,
 * because it provides a way to support two different types of input.
 *
 * Specializing `fanIn` to function application would look like this:
 *
 * ```purescript
 * fanIn :: forall a b c d. (a -> c) -> (b -> c) -> Either a b -> c
 * ```
 *
 * We take two functions, `f` and `g`, which both return the same type `c` and we transform them into a
 * single function which takes an `Either` value with the parameter type of `f` on the left side and
 * the parameter type of `g` on the right side. The function then runs either `f` or `g`, depending on
 * whether the `Either` value is a `Left` or a `Right`.
 * This allows us to bundle two different computations which both have the same result type into one
 * function which will run the appropriate computation based on the parameter supplied in the `Either` value.
 *
 * @since 3.0.0
 */
export const fanIn = (P, C) => {
    const splitPC = split(P, C);
    return (pac, pbc) => pipe(splitPC(pac, pbc), C.compose(pipe(C.id(), P.promap((cc) => (_.isLeft(cc) ? cc.left : cc.right), identity))));
};
