import { apFirst as apFirst_, apS as apS_, apSecond as apSecond_, apT as apT_ } from './Apply';
import { bind as bind_, chainFirst as chainFirst_ } from './Chain';
import * as E from './Either';
import { constant, flow, identity } from './function';
import { bindTo as bindTo_, flap as flap_, let as let__, tupled as tupled_ } from './Functor';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export const ask = () => identity;
/**
 * Projects a value from the global context in a `Reader`.
 *
 * @category constructors
 * @since 3.0.0
 */
export const asks = identity;
/**
 * @category constructors
 * @since 3.0.0
 */
export const asksReader = (f) => (r) => f(r)(r);
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export const local = (f) => (ma) => (r2) => ma(f(r2));
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export const map = (f) => (fa) => flow(fa, f);
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export const ap = (fa) => (fab) => (r) => fab(r)(fa(r));
/**
 * @category Pointed
 * @since 3.0.0
 */
export const of = constant;
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export const chain = (f) => (fa) => (r) => f(fa(r))(r);
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const flatten = /*#__PURE__*/ chain(identity);
/**
 * @category Semigroupoid
 * @since 3.0.0
 */
export const compose = (bc) => (ab) => flow(ab, bc);
/**
 * @category Profunctor
 * @since 3.0.0
 */
export const promap = (f, g) => (fea) => (a) => g(fea(f(a)));
/**
 * @category Category
 * @since 3.0.0
 */
export const id = () => identity;
/**
 * @category Choice
 * @since 3.0.0
 */
export const left = (pab) => E.match((a) => _.left(pab(a)), _.right);
/**
 * @category Choice
 * @since 3.0.0
 */
export const right = (pbc) => E.match(_.left, (b) => _.right(pbc(b)));
/**
 * @category Strong
 * @since 3.0.0
 */
export const first = (pab) => ([a, c]) => [
    pab(a),
    c
];
/**
 * @category Strong
 * @since 3.0.0
 */
export const second = (pbc) => ([a, b]) => [
    a,
    pbc(b)
];
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const FromReader = {
    fromReader: identity
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Pointed = {
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Apply = {
    map,
    ap
};
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apFirst = /*#__PURE__*/ apFirst_(Apply);
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const apSecond = /*#__PURE__*/ apSecond_(Apply);
/**
 * @category instances
 * @since 3.0.0
 */
export const Applicative = {
    map,
    ap,
    of
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Chain = {
    map,
    chain
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Monad = {
    map,
    of,
    chain
};
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export const chainFirst = /*#__PURE__*/ chainFirst_(Chain);
/**
 * @category instances
 * @since 3.0.0
 */
export const Profunctor = {
    map,
    promap
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Category = {
    compose,
    id
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Choice = {
    map,
    promap,
    left,
    right
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Strong = {
    map,
    promap,
    first,
    second
};
// -------------------------------------------------------------------------------------
// do notation
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const bindTo = /*#__PURE__*/ bindTo_(Functor);
const let_ = /*#__PURE__*/ let__(Functor);
export { 
/**
 * @since 3.0.0
 */
let_ as let };
/**
 * @since 3.0.0
 */
export const bind = /*#__PURE__*/ bind_(Chain);
// -------------------------------------------------------------------------------------
// sequence S
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const Do = /*#__PURE__*/ of(_.emptyRecord);
/**
 * @since 3.0.0
 */
export const apS = /*#__PURE__*/ apS_(Apply);
// -------------------------------------------------------------------------------------
// sequence T
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const ApT = /*#__PURE__*/ of(_.emptyReadonlyArray);
/**
 * @since 3.0.0
 */
export const tupled = /*#__PURE__*/ tupled_(Functor);
/**
 * @since 3.0.0
 */
export const apT = /*#__PURE__*/ apT_(Apply);
// -------------------------------------------------------------------------------------
// array utils
// -------------------------------------------------------------------------------------
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyNonEmptyArrayWithIndex = (f) => (as) => (r) => {
    const out = [f(0, _.head(as))(r)];
    for (let i = 1; i < as.length; i++) {
        out.push(f(i, as[i])(r));
    }
    return out;
};
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export const traverseReadonlyArrayWithIndex = (f) => {
    const g = traverseReadonlyNonEmptyArrayWithIndex(f);
    return (as) => (_.isNonEmpty(as) ? g(as) : ApT);
};
