import { identity, pipe } from './function';
import { flap as flap_ } from './Functor';
import * as _ from './internal';
// -------------------------------------------------------------------------------------
// constructors
// -------------------------------------------------------------------------------------
/**
 * @category constructors
 * @since 3.0.0
 */
export const fromIdentity = (w) => (a) => [a, w]; // TODO name?
/**
 * Appends a value to the accumulator
 *
 * @category constructors
 * @since 3.0.0
 */
export const tell = (w) => [undefined, w];
// -------------------------------------------------------------------------------------
// utils
// -------------------------------------------------------------------------------------
/**
 * @since 3.0.0
 */
export const fst = (t) => t[0];
/**
 * @since 3.0.0
 */
export const snd = (t) => t[1];
/**
 * Alias of [`fst`](#fst).
 *
 * @since 3.0.0
 */
export const evaluate = fst;
/**
 * Alias of [`snd`](#snd).
 *
 * @since 3.0.0
 */
export const execute = snd;
// -------------------------------------------------------------------------------------
// combinators
// -------------------------------------------------------------------------------------
/**
 * @category combinators
 * @since 3.0.0
 */
export const swap = (t) => [snd(t), fst(t)];
/**
 * Modifies the result to include the changes to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export const listen = (fa) => {
    const [a, w] = fa;
    return [[a, w], w];
};
/**
 * Applies the returned function to the accumulator
 *
 * @category combinators
 * @since 3.0.0
 */
export const pass = (fa) => {
    const [[a, f], w] = fa;
    return [a, f(w)];
};
/**
 * Projects a value from modifications made to the accumulator during an action
 *
 * @category combinators
 * @since 3.0.0
 */
export const listens = (f) => (fa) => {
    const [a, w] = fa;
    return [[a, f(w)], w];
};
/**
 * Modify the final accumulator value by applying a function
 *
 * @category combinators
 * @since 3.0.0
 */
export const censor = (f) => (fa) => {
    const [a, w] = fa;
    return [a, f(w)];
};
// -------------------------------------------------------------------------------------
// type class operations
// -------------------------------------------------------------------------------------
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const map = (f) => (fa) => {
    const [a, w] = fa;
    return [f(a), w];
};
/**
 * @category type class operations
 * @since 3.0.0
 */
export const mapLeft = (f) => (fa) => {
    const [a, w] = fa;
    return [a, f(w)];
};
/**
 * Map a pair of functions over the two type arguments of the bifunctor.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const bimap = (mapSnd, mapFst) => (t) => [
    mapFst(fst(t)),
    mapSnd(snd(t))
];
/**
 * Maps a function over the first component of a `Writer`.
 *
 * Alias of [`map`](#map)
 *
 * @since 3.0.0
 */
export const mapFst = map;
/**
 * Maps a function over the second component of a `Writer`.
 *
 * Alias of [`mapLeft`](#mapleft)
 *
 * @since 3.0.0
 */
export const mapSnd = mapLeft;
/**
 * @category type class operations
 * @since 3.0.0
 */
export const compose = (bc) => (ab) => [
    fst(bc),
    snd(ab)
];
/**
 * @category type class operations
 * @since 3.0.0
 */
export const extend = (f) => (wa) => [
    f(wa),
    snd(wa)
];
/**
 * @category type class operations
 * @since 3.0.0
 */
export const extract = fst;
/**
 * Derivable from `Extend`.
 *
 * @category type class operations
 * @since 3.0.0
 */
export const duplicate = /*#__PURE__*/ extend(identity);
/**
 * @category type class operations
 * @since 3.0.0
 */
export const reduce = (b, f) => (fa) => f(b, fst(fa));
/**
 * @category type class operations
 * @since 3.0.0
 */
export const foldMap = () => (f) => (fa) => f(fst(fa));
/**
 * @category type class operations
 * @since 3.0.0
 */
export const reduceRight = (b, f) => (fa) => f(fst(fa), b);
/**
 * @category type class operations
 * @since 3.0.0
 */
export const traverse = (F) => (f) => (t) => pipe(f(fst(t)), F.map((b) => [b, snd(t)]));
// -------------------------------------------------------------------------------------
// instances
// -------------------------------------------------------------------------------------
/**
 * @category instances
 * @since 3.0.0
 */
export const Bifunctor = {
    bimap,
    mapLeft: mapSnd
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Functor = {
    map: mapFst
};
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export const flap = /*#__PURE__*/ flap_(Functor);
/**
 * @category instances
 * @since 3.0.0
 */
export const Semigroupoid = {
    compose
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Comonad = {
    map: mapFst,
    extend,
    extract
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Foldable = {
    reduce,
    foldMap,
    reduceRight
};
/**
 * @category instances
 * @since 3.0.0
 */
export const Traversable = {
    map: mapFst,
    traverse
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getPointed = (M) => ({
    of: (a) => [a, M.empty]
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApply = (S) => ({
    map,
    ap: (fa) => (fab) => {
        const [f, w1] = fab;
        const [a, w2] = fa;
        return [f(a), S.concat(w2)(w1)];
    }
});
/**
 * @category instances
 * @since 3.0.0
 */
export const getApplicative = (M) => {
    const A = getApply(M);
    const P = getPointed(M);
    return {
        map,
        ap: A.ap,
        of: P.of
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getChain = (S) => {
    return {
        map,
        chain: (f) => (ma) => {
            const [a, w1] = ma;
            const [b, w2] = f(a);
            return [b, S.concat(w2)(w1)];
        }
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export const getMonad = (M) => {
    const P = getPointed(M);
    const C = getChain(M);
    return {
        map,
        of: P.of,
        chain: C.chain
    };
};
/**
 * @category instances
 * @since 3.0.0
 */
export function getChainRec(M) {
    const chainRec = (f) => (a) => {
        let result = f(a);
        let acc = M.empty;
        let s = fst(result);
        while (_.isLeft(s)) {
            acc = M.concat(snd(result))(acc);
            result = f(s.left);
            s = fst(result);
        }
        return [s.right, M.concat(snd(result))(acc)];
    };
    return {
        chainRec
    };
}
