/**
 * `Filterable` represents data structures which can be _partitioned_/_filtered_.
 *
 * @since 3.0.0
 */
import type { Either } from './Either'
import type { Functor } from './Functor'
import type { HKT, Kind, Typeclass } from './HKT'
import { Option } from './Option'
import { Predicate } from './Predicate'
import type { Refinement } from './Refinement'
import { Separated } from './Separated'
/**
 * @category type classes
 * @since 3.0.0
 */
export interface Filterable<F extends HKT> extends Typeclass<F> {
  readonly partitionMap: <A, B, C>(
    f: (a: A) => Either<B, C>
  ) => <S, R, W, E>(fa: Kind<F, S, R, W, E, A>) => Separated<Kind<F, S, R, W, E, B>, Kind<F, S, R, W, E, C>>
  readonly partition: {
    <A, B extends A>(refinement: Refinement<A, B>): <S, R, W, E>(
      fa: Kind<F, S, R, W, E, A>
    ) => Separated<Kind<F, S, R, W, E, A>, Kind<F, S, R, W, E, B>>
    <A>(predicate: Predicate<A>): <S, R, W, E, B extends A>(
      fb: Kind<F, S, R, W, E, B>
    ) => Separated<Kind<F, S, R, W, E, B>, Kind<F, S, R, W, E, B>>
    <A>(predicate: Predicate<A>): <S, R, W, E>(
      fa: Kind<F, S, R, W, E, A>
    ) => Separated<Kind<F, S, R, W, E, A>, Kind<F, S, R, W, E, A>>
  }
  readonly filterMap: <A, B>(
    f: (a: A) => Option<B>
  ) => <S, R, W, E>(fa: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, B>
  readonly filter: {
    <A, B extends A>(refinement: Refinement<A, B>): <S, R, W, E>(fa: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, B>
    <A>(predicate: Predicate<A>): <S, R, W, E, B extends A>(fb: Kind<F, S, R, W, E, B>) => Kind<F, S, R, W, E, B>
    <A>(predicate: Predicate<A>): <S, R, W, E>(fa: Kind<F, S, R, W, E, A>) => Kind<F, S, R, W, E, A>
  }
}
/**
 * `filter` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const filter: <F extends HKT, G extends HKT>(
  F: Functor<F>,
  G: Filterable<G>
) => {
  <A, B extends A>(refinement: Refinement<A, B>): <FS, FR, FW, FE, GS, GR, GW, GE>(
    fga: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>
  ) => Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, B>>
  <A_1>(predicate: Predicate<A_1>): <FS_1, FR_1, FW_1, FE_1, GS_1, GR_1, GW_1, GE_1, B_1 extends A_1>(
    fgb: Kind<F, FS_1, FR_1, FW_1, FE_1, Kind<G, GS_1, GR_1, GW_1, GE_1, B_1>>
  ) => Kind<F, FS_1, FR_1, FW_1, FE_1, Kind<G, GS_1, GR_1, GW_1, GE_1, B_1>>
  <A_2>(predicate: Predicate<A_2>): <FS_2, FR_2, FW_2, FE_2, GS_2, GR_2, GW_2, GE_2>(
    fga: Kind<F, FS_2, FR_2, FW_2, FE_2, Kind<G, GS_2, GR_2, GW_2, GE_2, A_2>>
  ) => Kind<F, FS_2, FR_2, FW_2, FE_2, Kind<G, GS_2, GR_2, GW_2, GE_2, A_2>>
}
/**
 * `filterMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const filterMap: <F extends HKT, G extends HKT>(
  F: Functor<F>,
  G: Filterable<G>
) => <A, B>(
  f: (a: A) => Option<B>
) => <FS, FR, FW, FE, GS, GR, GW, GE>(
  fga: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>
) => Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, B>>
/**
 * `partition` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const partition: <F extends HKT, G extends HKT>(
  F: Functor<F>,
  G: Filterable<G>
) => {
  <A, B extends A>(refinement: Refinement<A, B>): <FS, FR, FW, FE, GS, GR, GW, GE>(
    fga: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>
  ) => Separated<
    Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>,
    Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, B>>
  >
  <A_1>(predicate: Predicate<A_1>): <FS_1, FR_1, FW_1, FE_1, GS_1, GR_1, GW_1, GE_1, B_1 extends A_1>(
    fgb: Kind<F, FS_1, FR_1, FW_1, FE_1, Kind<G, GS_1, GR_1, GW_1, GE_1, B_1>>
  ) => Separated<
    Kind<F, FS_1, FR_1, FW_1, FE_1, Kind<G, GS_1, GR_1, GW_1, GE_1, B_1>>,
    Kind<F, FS_1, FR_1, FW_1, FE_1, Kind<G, GS_1, GR_1, GW_1, GE_1, B_1>>
  >
  <A_2>(predicate: Predicate<A_2>): <FS_2, FR_2, FW_2, FE_2, GS_2, GR_2, GW_2, GE_2>(
    fga: Kind<F, FS_2, FR_2, FW_2, FE_2, Kind<G, GS_2, GR_2, GW_2, GE_2, A_2>>
  ) => Separated<
    Kind<F, FS_2, FR_2, FW_2, FE_2, Kind<G, GS_2, GR_2, GW_2, GE_2, A_2>>,
    Kind<F, FS_2, FR_2, FW_2, FE_2, Kind<G, GS_2, GR_2, GW_2, GE_2, A_2>>
  >
}
/**
 * `partitionMap` composition.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const partitionMap: <F extends HKT, G extends HKT>(
  F: Functor<F>,
  G: Filterable<G>
) => <A, B, C>(
  f: (a: A) => Either<B, C>
) => <FS, FR, FW, FE, GS, GR, GW, GE>(
  fga: Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, A>>
) => Separated<Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, B>>, Kind<F, FS, FR, FW, FE, Kind<G, GS, GR, GW, GE, C>>>
