/**
 * `IOOption<A>` represents a synchronous computation that either yields a value of type `A` or nothing.
 *
 * If you want to represent a synchronous computation that never fails, please see `IO`.
 * If you want to represent a synchronous computation that may fail, please see `IOEither`.
 *
 * @since 3.0.0
 */
import { Alt as Alt_ } from './Alt'
import { Alternative as Alternative_ } from './Alternative'
import { Applicative as Applicative_ } from './Applicative'
import { Apply as Apply_ } from './Apply'
import { Chain as Chain_ } from './Chain'
import { Compactable as Compactable_ } from './Compactable'
import { Either } from './Either'
import { Filterable as Filterable_ } from './Filterable'
import { FromEither as FromEither_ } from './FromEither'
import { FromIO as FromIO_ } from './FromIO'
import { Lazy } from './function'
import { Functor as Functor_ } from './Functor'
import { HKT } from './HKT'
import * as I from './IO'
import { IOEither } from './IOEither'
import { Monad as Monad_ } from './Monad'
import * as O from './Option'
import { Pointed as Pointed_ } from './Pointed'
import { Predicate } from './Predicate'
import { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import { Refinement } from './Refinement'
import { Separated } from './Separated'
import { Zero as Zero_ } from './Zero'
import IO = I.IO
import Option = O.Option
/**
 * @category model
 * @since 3.0.0
 */
export interface IOOption<A> extends IO<Option<A>> {}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const some: <A>(a: A) => IOOption<A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const fromPredicate: {
  <A, B extends A>(refinement: Refinement<A, B>): (a: A) => IOOption<B>
  <A>(predicate: Predicate<A>): <B extends A>(b: B) => IOOption<B>
  <A>(predicate: Predicate<A>): (a: A) => IOOption<A>
}
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromOption: <A>(fa: Option<A>) => IOOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromEither: <A>(e: Either<unknown, A>) => I.IO<O.Option<A>>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIO: <A>(ma: I.IO<A>) => IOOption<A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIOEither: <A>(ma: IOEither<unknown, A>) => IOOption<A>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const match: <B, A, C = B>(onNone: () => B, onSome: (a: A) => C) => (ma: IOOption<A>) => IO<B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const matchE: <B, A, C = B>(
  onNone: () => IO<B>,
  onSome: (a: A) => IO<C>
) => (ma: IOOption<A>) => IO<B | C>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElse: <B>(onNone: Lazy<B>) => <A>(ma: IOOption<A>) => IO<A | B>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const getOrElseE: <B>(onNone: Lazy<IO<B>>) => <A>(ma: IOOption<A>) => IO<A | B>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const toUndefined: <A>(ma: IOOption<A>) => IO<A | undefined>
/**
 * @category destructors
 * @since 3.0.0
 */
export declare const toNullable: <A>(ma: IOOption<A>) => IO<A | null>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullable: <A>(a: A) => IOOption<NonNullable<A>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const fromNullableK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => B | null | undefined
) => (...a: A) => IOOption<NonNullable<B>>
/**
 * @category interop
 * @since 3.0.0
 */
export declare const chainNullableK: <A, B>(
  f: (a: A) => B | null | undefined
) => (ma: IOOption<A>) => IOOption<NonNullable<B>>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromOptionK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => Option<B>
) => (...a: A) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainOptionK: <A, B>(f: (a: A) => Option<B>) => (ma: IOOption<A>) => IOOption<B>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: IOOption<A>) => IOOption<B>
/**
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <A>(fa: IOOption<A>) => <B>(fab: IOOption<(a: A) => B>) => IOOption<B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => IOOption<A>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: <A, B>(f: (a: A) => IOOption<B>) => (ma: IOOption<A>) => IOOption<B>
/**
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: IOOption<IOOption<A>>) => IOOption<A>
/**
 * @category Alt
 * @since 3.0.0
 */
export declare const alt: <B>(second: Lazy<IOOption<B>>) => <A>(first: IOOption<A>) => IOOption<A | B>
/**
 * @category Zero
 * @since 3.0.0
 */
export declare const zero: <A>() => IOOption<A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const none: IOOption<never>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const compact: <A>(foa: IOOption<O.Option<A>>) => IOOption<A>
/**
 * @category Compactable
 * @since 3.0.0
 */
export declare const separate: <A, B>(fe: IOOption<Either<A, B>>) => Separated<IOOption<A>, IOOption<B>>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filter: {
  <A, B extends A>(refinement: Refinement<A, B>): (fb: IOOption<A>) => IOOption<B>
  <A>(predicate: Predicate<A>): <B extends A>(fb: IOOption<B>) => IOOption<B>
  <A>(predicate: Predicate<A>): (fa: IOOption<A>) => IOOption<A>
}
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const filterMap: <A, B>(f: (a: A) => Option<B>) => (fga: IOOption<A>) => IOOption<B>
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partition: {
  <A, B extends A>(refinement: Refinement<A, B>): (fb: IOOption<A>) => Separated<IOOption<A>, IOOption<B>>
  <A>(predicate: Predicate<A>): <B extends A>(fb: IOOption<B>) => Separated<IOOption<B>, IOOption<B>>
  <A>(predicate: Predicate<A>): (fa: IOOption<A>) => Separated<IOOption<A>, IOOption<A>>
}
/**
 * @category Filterable
 * @since 3.0.0
 */
export declare const partitionMap: <A, B, C>(
  f: (a: A) => Either<B, C>
) => (fa: IOOption<A>) => Separated<IOOption<B>, IOOption<C>>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface IOOptionF extends HKT {
  readonly type: IOOption<this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<IOOptionF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: IOOption<(a: A) => B>) => IOOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed_<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply_<IOOptionF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const apFirst: <B>(second: IOOption<B>) => <A>(first: IOOption<A>) => IOOption<A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const apSecond: <B>(second: IOOption<B>) => <A>(first: IOOption<A>) => IOOption<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative_<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain_<IOOptionF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, B>(f: (a: A) => IOOption<B>) => (first: IOOption<A>) => IOOption<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alt: Alt_<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Zero: Zero_<IOOptionF>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const guard: (b: boolean) => IOOption<void>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alternative: Alternative_<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad_<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Compactable: Compactable_<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Filterable: Filterable_<IOOptionF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: FromIO_<IOOptionF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(f: (...a: A) => I.IO<B>) => (...a: A) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainIOK: <A, B>(f: (a: A) => I.IO<B>) => (first: IOOption<A>) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstIOK: <A, B>(f: (a: A) => I.IO<B>) => (first: IOOption<A>) => IOOption<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromEither: FromEither_<IOOptionF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromEitherK: <A extends ReadonlyArray<unknown>, E, B>(
  f: (...a: A) => Either<E, B>
) => (...a: A) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainEitherK: <A, E, B>(f: (a: A) => Either<E, B>) => (ma: IOOption<A>) => IOOption<B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstEitherK: <A, E, B>(f: (a: A) => Either<E, B>) => (ma: IOOption<A>) => IOOption<A>
/**
 * @since 3.0.0
 */
export declare const Do: IOOption<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(
  fa: IOOption<A>
) => IOOption<
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (
  fa: IOOption<A>
) => IOOption<
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => IOOption<B>
) => (
  ma: IOOption<A>
) => IOOption<
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: IOOption<B>
) => (
  fa: IOOption<A>
) => IOOption<
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const ApT: IOOption<readonly []>
/**
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => IOOption<B>
) => (as: ReadonlyNonEmptyArray<A>) => IOOption<ReadonlyNonEmptyArray<B>>
/**
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, B>(
  f: (index: number, a: A) => IOOption<B>
) => (as: readonly A[]) => IOOption<readonly B[]>
