/**
 * @since 3.0.0
 */
import type { Alt as Alt_ } from './Alt'
import type { Applicative as Applicative_ } from './Applicative'
import { Apply as Apply_ } from './Apply'
import { Chain as Chain_ } from './Chain'
import { ChainRec as ChainRec_ } from './ChainRec'
import type { Comonad as Comonad_ } from './Comonad'
import { Either } from './Either'
import type { Eq } from './Eq'
import type { Foldable as Foldable_ } from './Foldable'
import { Functor as Functor_ } from './Functor'
import type { HKT, Kind } from './HKT'
import type { Monad as Monad_ } from './Monad'
import type { Monoid } from './Monoid'
import type { Pointed as Pointed_ } from './Pointed'
import type { Show } from './Show'
import type { Traversable as Traversable_ } from './Traversable'
/**
 * @category model
 * @since 3.0.0
 */
export declare type Identity<A> = A
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: Identity<A>) => Identity<B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <A>(fa: Identity<A>) => <B>(fab: Identity<(a: A) => B>) => Identity<B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => Identity<A>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: <A, B>(f: (a: A) => Identity<B>) => (ma: Identity<A>) => Identity<B>
/**
 * @category Extend
 * @since 3.0.0
 */
export declare const extend: <A, B>(f: (wa: Identity<A>) => B) => (wa: Identity<A>) => B
/**
 * @category Extract
 * @since 3.0.0
 */
export declare const extract: <A>(wa: Identity<A>) => A
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const duplicate: <A>(ma: Identity<A>) => Identity<Identity<A>>
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: Identity<Identity<A>>) => Identity<A>
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduce: <B, A>(b: B, f: (b: B, a: A) => B) => (fa: Identity<A>) => B
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const foldMap: <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: Identity<A>) => M
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduceRight: <B, A>(b: B, f: (a: A, b: B) => B) => (fa: Identity<A>) => B
/**
 * @since 3.0.0
 */
export declare const traverse: <F extends HKT>(
  F: Applicative_<F>
) => <A, S, R, W, E, B>(f: (a: A) => Kind<F, S, R, W, E, B>) => (ta: Identity<A>) => Kind<F, S, R, W, E, B>
/**
 * Identifies an associative operation on a type constructor. It is similar to `Semigroup`, except that it applies to
 * types of kind `* -> *`.
 *
 * @category Alt
 * @since 3.0.0
 */
export declare const alt: <B>(second: () => Identity<B>) => <A>(first: Identity<A>) => Identity<A | B>
/**
 * @category ChainRec
 * @since 3.0.0
 */
export declare const chainRec: <A, B>(f: (a: A) => Identity<Either<A, B>>) => (a: A) => B
/**
 * @category HKT
 * @since 3.0.0
 */
export interface IdentityF extends HKT {
  readonly type: Identity<this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <A>(S: Show<A>) => Show<Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <A>(E: Eq<A>) => Eq<Identity<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<IdentityF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: Identity<(a: A) => B>) => B
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed_<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply_<IdentityF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <B>(second: Identity<B>) => <A>(first: Identity<A>) => Identity<A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <B>(second: Identity<B>) => <A>(first: Identity<A>) => Identity<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative_<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain_<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad_<IdentityF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, B>(f: (a: A) => Identity<B>) => (first: Identity<A>) => Identity<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const ChainRec: ChainRec_<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: Foldable_<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: Traversable_<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Alt: Alt_<IdentityF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: Comonad_<IdentityF>
/**
 * @since 3.0.0
 */
export declare const Do: Identity<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(
  fa: Identity<A>
) => {
  readonly [K in N]: A
}
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (
  fa: A
) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => Identity<B>
) => (
  ma: Identity<A>
) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: Identity<B>
) => (
  fa: Identity<A>
) => {
  readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
}
/**
 * @since 3.0.0
 */
export declare const ApT: Identity<readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <A>(fa: Identity<A>) => readonly [A]
/**
 * @since 3.0.0
 */
export declare const apT: <B>(fb: B) => <A extends ReadonlyArray<unknown>>(fas: A) => readonly [...A, B]
