/**
 * @since 3.0.0
 */
import type { Applicative as Applicative_ } from './Applicative'
import { Apply as Apply_ } from './Apply'
import type { Category as Category_ } from './Category'
import { Chain as Chain_ } from './Chain'
import type { Choice as Choice_ } from './Choice'
import * as E from './Either'
import type { FromReader as FromReader_ } from './FromReader'
import { Functor as Functor_ } from './Functor'
import { HKT } from './HKT'
import type { Monad as Monad_ } from './Monad'
import type { Pointed as Pointed_ } from './Pointed'
import type { Profunctor as Profunctor_ } from './Profunctor'
import type { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
import type { Strong as Strong_ } from './Strong'
/**
 * @category model
 * @since 3.0.0
 */
export interface Reader<R, A> {
  (r: R): A
}
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const ask: <R>() => Reader<R, R>
/**
 * Projects a value from the global context in a `Reader`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const asks: <R, A>(f: (r: R) => A) => Reader<R, A>
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const asksReader: <R1, R2, A>(f: (r1: R1) => Reader<R2, A>) => Reader<R1 & R2, A>
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const local: <R2, R1>(f: (r2: R2) => R1) => <A>(ma: Reader<R1, A>) => Reader<R2, A>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <R>(fa: Reader<R, A>) => Reader<R, B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <R2, A>(fa: Reader<R2, A>) => <R1, B>(fab: Reader<R1, (a: A) => B>) => Reader<R1 & R2, B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A, R = unknown>(a: A) => Reader<R, A>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: <A, R2, B>(f: (a: A) => Reader<R2, B>) => <R1>(ma: Reader<R1, A>) => Reader<R1 & R2, B>
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <R1, R2, A>(mma: Reader<R1, Reader<R2, A>>) => Reader<R1 & R2, A>
/**
 * @category Semigroupoid
 * @since 3.0.0
 */
export declare const compose: <B, C>(bc: Reader<B, C>) => <A>(ab: Reader<A, B>) => Reader<A, C>
/**
 * @category Profunctor
 * @since 3.0.0
 */
export declare const promap: <Q, R, A, B>(f: (d: Q) => R, g: (a: A) => B) => (pea: Reader<R, A>) => Reader<Q, B>
/**
 * @category Category
 * @since 3.0.0
 */
export declare const id: <A>() => Reader<A, A>
/**
 * @category Choice
 * @since 3.0.0
 */
export declare const left: <A, B, C>(pab: Reader<A, B>) => Reader<E.Either<A, C>, E.Either<B, C>>
/**
 * @category Choice
 * @since 3.0.0
 */
export declare const right: <B, C, A>(pbc: Reader<B, C>) => Reader<E.Either<A, B>, E.Either<A, C>>
/**
 * @category Strong
 * @since 3.0.0
 */
export declare const first: <A, B, C>(pab: Reader<A, B>) => Reader<readonly [A, C], readonly [B, C]>
/**
 * @category Strong
 * @since 3.0.0
 */
export declare const second: <B, C, A>(pab: Reader<B, C>) => Reader<readonly [A, B], readonly [A, C]>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface ReaderF extends HKT {
  readonly type: Reader<this['Contravariant1'], this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromReader: FromReader_<ReaderF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<ReaderF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <R, B>(fab: Reader<R, (a: A) => B>) => Reader<R, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed_<ReaderF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply_<ReaderF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <R2, B>(second: Reader<R2, B>) => <R1, A>(first: Reader<R1, A>) => Reader<R1 & R2, A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <R2, B>(second: Reader<R2, B>) => <R1, A>(first: Reader<R1, A>) => Reader<R1 & R2, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative_<ReaderF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain_<ReaderF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad_<ReaderF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, R2, B>(f: (a: A) => Reader<R2, B>) => <R1>(ma: Reader<R1, A>) => Reader<R1 & R2, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Profunctor: Profunctor_<ReaderF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Category: Category_<ReaderF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Choice: Choice_<ReaderF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Strong: Strong_<ReaderF>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <R, A>(
  fa: Reader<R, A>
) => Reader<
  R,
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <R>(
  fa: Reader<R, A>
) => Reader<
  R,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, R2, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => Reader<R2, B>
) => <R1>(
  fa: Reader<R1, A>
) => Reader<
  R1 & R2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const Do: Reader<unknown, {}>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, R2, B>(
  name: Exclude<N, keyof A>,
  fb: Reader<R2, B>
) => <R1>(
  fa: Reader<R1, A>
) => Reader<
  R1 & R2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const ApT: Reader<unknown, readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <R, A>(fa: Reader<R, A>) => Reader<R, readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <R2, B>(
  fb: Reader<R2, B>
) => <R1, A extends ReadonlyArray<unknown>>(fas: Reader<R1, A>) => Reader<R1 & R2, readonly [...A, B]>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, R, B>(
  f: (index: number, a: A) => Reader<R, B>
) => (as: ReadonlyNonEmptyArray<A>) => Reader<R, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, R, B>(
  f: (index: number, a: A) => Reader<R, B>
) => (as: readonly A[]) => Reader<R, readonly B[]>
