/**
 * @since 3.0.0
 */
import { Applicative as Applicative_ } from './Applicative'
import { Apply as Apply_ } from './Apply'
import { Chain as Chain_ } from './Chain'
import { FromIO as FromIO_ } from './FromIO'
import { FromReader as FromReader_ } from './FromReader'
import { Functor as Functor_ } from './Functor'
import { HKT } from './HKT'
import * as I from './IO'
import { Monad as Monad_ } from './Monad'
import { Pointed as Pointed_ } from './Pointed'
import * as R from './Reader'
import { ReadonlyNonEmptyArray } from './ReadonlyNonEmptyArray'
/**
 * @category model
 * @since 3.0.0
 */
export interface ReaderIO<R, A> {
  (r: R): I.IO<A>
}
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromReader: <R, A>(fa: R.Reader<R, A>) => ReaderIO<R, A>
/**
 * @category natural transformations
 * @since 3.0.0
 */
export declare const fromIO: <A, R = unknown>(fa: I.IO<A>) => ReaderIO<R, A>
/**
 * Changes the value of the local context during the execution of the action `ma` (similar to `Contravariant`'s
 * `contramap`).
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const local: <R2, R1>(f: (r2: R2) => R1) => <A>(ma: ReaderIO<R1, A>) => ReaderIO<R2, A>
/**
 * Effectfully accesses the environment.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const asksReaderIO: <R1, R2, A>(f: (r1: R1) => ReaderIO<R2, A>) => ReaderIO<R1 & R2, A>
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => <R>(fa: ReaderIO<R, A>) => ReaderIO<R, B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <R2, A>(fa: ReaderIO<R2, A>) => <R1, B>(fab: ReaderIO<R1, (a: A) => B>) => ReaderIO<R1 & R2, B>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A, R = unknown>(a: A) => ReaderIO<R, A>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Monad
 * @since 3.0.0
 */
export declare const chain: <A, R2, B>(
  f: (a: A) => ReaderIO<R2, B>
) => <R1>(ma: ReaderIO<R1, A>) => ReaderIO<R1 & R2, B>
/**
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flatten: <R1, R2, A>(mma: ReaderIO<R1, ReaderIO<R2, A>>) => ReaderIO<R1 & R2, A>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface ReaderIOF extends HKT {
  readonly type: ReaderIO<this['Contravariant1'], this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<ReaderIOF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <R, B>(fab: ReaderIO<R, (a: A) => B>) => ReaderIO<R, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed_<ReaderIOF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply_<ReaderIOF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const apFirst: <R, B>(second: ReaderIO<R, B>) => <A>(first: ReaderIO<R, A>) => ReaderIO<R, A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const apSecond: <R, B>(second: ReaderIO<R, B>) => <A>(first: ReaderIO<R, A>) => ReaderIO<R, B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative_<ReaderIOF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain_<ReaderIOF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad_<ReaderIOF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, R2, B>(
  f: (a: A) => ReaderIO<R2, B>
) => <R1>(ma: ReaderIO<R1, A>) => ReaderIO<R1 & R2, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromIO: FromIO_<ReaderIOF>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromIOK: <A extends ReadonlyArray<unknown>, B>(
  f: (...a: A) => I.IO<B>
) => <R = unknown>(...a: A) => ReaderIO<R, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainIOK: <A, B>(f: (a: A) => I.IO<B>) => <R>(first: ReaderIO<R, A>) => ReaderIO<R, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstIOK: <A, B>(f: (a: A) => I.IO<B>) => <R>(first: ReaderIO<R, A>) => ReaderIO<R, A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const FromReader: FromReader_<ReaderIOF>
/**
 * Reads the current context.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const ask: <R>() => ReaderIO<R, R>
/**
 * Projects a value from the global context in a `ReaderIO`.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const asks: <R, A>(f: (r: R) => A) => ReaderIO<R, A>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const fromReaderK: <A extends ReadonlyArray<unknown>, R, B>(
  f: (...a: A) => R.Reader<R, B>
) => (...a: A) => ReaderIO<R, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainReaderK: <A, R2, B>(
  f: (a: A) => R.Reader<R2, B>
) => <R1>(ma: ReaderIO<R1, A>) => ReaderIO<R1 & R2, B>
/**
 * @category combinators
 * @since 3.0.0
 */
export declare const chainFirstReaderK: <A, R2, B>(
  f: (a: A) => R.Reader<R2, B>
) => <R1>(ma: ReaderIO<R1, A>) => ReaderIO<R1 & R2, A>
/**
 * @since 3.0.0
 */
export declare const Do: ReaderIO<unknown, {}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <R, A>(
  fa: ReaderIO<R, A>
) => ReaderIO<
  R,
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => <R>(
  fa: ReaderIO<R, A>
) => ReaderIO<
  R,
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, R2, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => ReaderIO<R2, B>
) => <R1>(
  fa: ReaderIO<R1, A>
) => ReaderIO<
  R1 & R2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, R2, B>(
  name: Exclude<N, keyof A>,
  fb: ReaderIO<R2, B>
) => <R1>(
  fa: ReaderIO<R1, A>
) => ReaderIO<
  R1 & R2,
  {
    readonly [K in keyof A | N]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const ApT: ReaderIO<unknown, readonly []>
/**
 * Equivalent to `ReadonlyNonEmptyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyNonEmptyArrayWithIndex: <A, R, B>(
  f: (index: number, a: A) => ReaderIO<R, B>
) => (as: ReadonlyNonEmptyArray<A>) => ReaderIO<R, ReadonlyNonEmptyArray<B>>
/**
 * Equivalent to `ReadonlyArray#traverseWithIndex(Applicative)`.
 *
 * @since 3.0.0
 */
export declare const traverseReadonlyArrayWithIndex: <A, R, B>(
  f: (index: number, a: A) => ReaderIO<R, B>
) => (as: readonly A[]) => ReaderIO<R, readonly B[]>
