/**
 * Multi-way trees (aka rose trees) and forests, where a forest is
 *
 * ```ts
 * type Forest<A> = ReadonlyArray<Tree<A>>
 * ```
 *
 * @since 3.0.0
 */
import type { Applicative as Applicative_ } from './Applicative'
import { Apply as Apply_ } from './Apply'
import { Chain as Chain_ } from './Chain'
import type { Comonad as Comonad_ } from './Comonad'
import { Eq } from './Eq'
import type { Foldable as Foldable_ } from './Foldable'
import { Functor as Functor_ } from './Functor'
import type { HKT, Kind } from './HKT'
import type { Monad as Monad_ } from './Monad'
import type { Monoid } from './Monoid'
import type { Pointed as Pointed_ } from './Pointed'
import { Predicate } from './Predicate'
import type { Show } from './Show'
import type { Traversable as Traversable_ } from './Traversable'
/**
 * @category model
 * @since 3.0.0
 */
export interface Forest<A> extends ReadonlyArray<Tree<A>> {}
/**
 * @category model
 * @since 3.0.0
 */
export interface Tree<A> {
  readonly value: A
  readonly forest: Forest<A>
}
/**
 * @category constructors
 * @since 3.0.0
 */
export declare const tree: <A>(value: A, forest?: Forest<A>) => Tree<A>
/**
 * Build a (possibly infinite) tree from a seed value in breadth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const unfoldTree: <B, A>(f: (b: B) => readonly [A, readonly B[]]) => (b: B) => Tree<A>
/**
 * Build a (possibly infinite) forest from a list of seed values in breadth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare const unfoldForest: <B, A>(f: (b: B) => readonly [A, readonly B[]]) => (bs: readonly B[]) => Forest<A>
/**
 * Monadic tree builder, in depth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare function unfoldTreeM<M extends HKT>(
  M: Monad_<M>,
  A: Applicative_<M>
): <B, S, R, W, E, A>(
  f: (b: B) => Kind<M, S, R, W, E, readonly [A, ReadonlyArray<B>]>
) => (b: B) => Kind<M, S, R, W, E, Tree<A>>
/**
 * Monadic forest builder, in depth-first order.
 *
 * @category constructors
 * @since 3.0.0
 */
export declare function unfoldForestM<M extends HKT>(
  M: Monad_<M>,
  A: Applicative_<M>
): <B, S, R, W, E, A>(
  f: (b: B) => Kind<M, S, R, W, E, readonly [A, ReadonlyArray<B>]>
) => (bs: ReadonlyArray<B>) => Kind<M, S, R, W, E, Forest<A>>
/**
 * Fold a tree into a "summary" value in depth-first order.
 *
 * For each node in the tree, apply `f` to the `value` and the result of applying `f` to each `forest`.
 *
 * This is also known as the catamorphism on trees.
 *
 * @example
 * import { fold, tree } from 'fp-ts/Tree'
 * import * as N from 'fp-ts/number'
 * import { concatAll } from 'fp-ts/Monoid'
 * import { pipe } from 'fp-ts/function'
 * import { isEmpty } from 'fp-ts/ReadonlyArray'
 *
 * const t = tree(1, [tree(2), tree(3)])
 *
 * const sum = concatAll(N.MonoidSum)
 *
 * assert.deepStrictEqual(pipe(t, fold((a, bs) => a + sum(bs))), 6)
 * assert.deepStrictEqual(pipe(t, fold((a, bs) => bs.reduce((b, acc) => Math.max(b, acc), a))), 3)
 * assert.deepStrictEqual(pipe(t, fold((_, bs) => (isEmpty(bs) ? 1 : sum(bs)))), 2)
 *
 * @category destructors
 * @since 3.0.0
 */
export declare const fold: <A, B>(f: (a: A, bs: readonly B[]) => B) => (tree: Tree<A>) => B
/**
 * `map` can be used to turn functions `(a: A) => B` into functions `(fa: F<A>) => F<B>` whose argument and return types
 * use the type constructor `F` to represent some computational context.
 *
 * @category Functor
 * @since 3.0.0
 */
export declare const map: <A, B>(f: (a: A) => B) => (fa: Tree<A>) => Tree<B>
/**
 * Apply a function to an argument under a type constructor.
 *
 * @category Apply
 * @since 3.0.0
 */
export declare const ap: <A>(fa: Tree<A>) => <B>(fab: Tree<(a: A) => B>) => Tree<B>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation.
 *
 * @category Chain
 * @since 3.0.0
 */
export declare const chain: <A, B>(f: (a: A) => Tree<B>) => (ma: Tree<A>) => Tree<B>
/**
 * @category Extend
 * @since 3.0.0
 */
export declare const extend: <A, B>(f: (wa: Tree<A>) => B) => (wa: Tree<A>) => Tree<B>
/**
 * Derivable from `Extend`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const duplicate: <A>(wa: Tree<A>) => Tree<Tree<A>>
/**
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const flatten: <A>(mma: Tree<Tree<A>>) => Tree<A>
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduce: <B, A>(b: B, f: (b: B, a: A) => B) => (fa: Tree<A>) => B
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const foldMap: <M>(M: Monoid<M>) => <A>(f: (a: A) => M) => (fa: Tree<A>) => M
/**
 * @category Foldable
 * @since 3.0.0
 */
export declare const reduceRight: <B, A>(b: B, f: (a: A, b: B) => B) => (fa: Tree<A>) => B
/**
 * @category Extract
 * @since 3.0.0
 */
export declare const extract: <A>(wa: Tree<A>) => A
/**
 * @since 3.0.0
 */
export declare const traverse: <F extends HKT>(
  F: Applicative_<F>
) => <A, S, R, W, E, B>(f: (a: A) => Kind<F, S, R, W, E, B>) => (ta: Tree<A>) => Kind<F, S, R, W, E, Tree<B>>
/**
 * @category Pointed
 * @since 3.0.0
 */
export declare const of: <A>(a: A) => Tree<A>
/**
 * @category HKT
 * @since 3.0.0
 */
export interface TreeF extends HKT {
  readonly type: Tree<this['Covariant1']>
}
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getShow: <A>(S: Show<A>) => Show<Tree<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const getEq: <A>(E: Eq<A>) => Eq<Tree<A>>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Functor: Functor_<TreeF>
/**
 * Derivable from `Functor`.
 *
 * @category combinators
 * @since 3.0.0
 */
export declare const flap: <A>(a: A) => <B>(fab: Tree<(a: A) => B>) => Tree<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Pointed: Pointed_<TreeF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Apply: Apply_<TreeF>
/**
 * Combine two effectful actions, keeping only the result of the first.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apFirst: <B>(second: Tree<B>) => <A>(first: Tree<A>) => Tree<A>
/**
 * Combine two effectful actions, keeping only the result of the second.
 *
 * Derivable from `Apply`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const apSecond: <B>(second: Tree<B>) => <A>(first: Tree<A>) => Tree<B>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Applicative: Applicative_<TreeF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Chain: Chain_<TreeF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Monad: Monad_<TreeF>
/**
 * Composes computations in sequence, using the return value of one computation to determine the next computation and
 * keeping only the result of the first.
 *
 * Derivable from `Chain`.
 *
 * @category derivable combinators
 * @since 3.0.0
 */
export declare const chainFirst: <A, B>(f: (a: A) => Tree<B>) => (first: Tree<A>) => Tree<A>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Foldable: Foldable_<TreeF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Traversable: Traversable_<TreeF>
/**
 * @category instances
 * @since 3.0.0
 */
export declare const Comonad: Comonad_<TreeF>
/**
 * @since 3.0.0
 */
export declare const exists: <A>(predicate: Predicate<A>) => (ma: Tree<A>) => boolean
/**
 * Tests whether a value is a member of a `Tree`.
 *
 * @since 3.0.0
 */
export declare const elem: <A>(E: Eq<A>) => (a: A) => (fa: Tree<A>) => boolean
/**
 * Neat 2-dimensional drawing of a forest
 *
 * @since 3.0.0
 */
export declare const drawForest: (forest: Forest<string>) => string
/**
 * Neat 2-dimensional drawing of a tree
 *
 * @example
 * import { tree, drawTree } from 'fp-ts/Tree'
 *
 * const fa = tree('a', [
 *   tree('b'),
 *   tree('c'),
 *   tree('d', [tree('e'), tree('f')])
 * ])
 *
 * assert.strictEqual(drawTree(fa), `a
 * ├─ b
 * ├─ c
 * └─ d
 *    ├─ e
 *    └─ f`)
 *
 *
 * @since 3.0.0
 */
export declare const drawTree: (tree: Tree<string>) => string
/**
 * @since 3.0.0
 */
export declare const Do: Tree<{}>
/**
 * @since 3.0.0
 */
export declare const bindTo: <N extends string>(
  name: N
) => <A>(
  fa: Tree<A>
) => Tree<
  {
    readonly [K in N]: A
  }
>
declare const let_: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: (a: A) => B
) => (
  fa: Tree<A>
) => Tree<
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
export {
  /**
   * @since 3.0.0
   */
  let_ as let
}
/**
 * @since 3.0.0
 */
export declare const bind: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  f: <A2 extends A>(a: A | A2) => Tree<B>
) => (
  ma: Tree<A>
) => Tree<
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const apS: <N extends string, A, B>(
  name: Exclude<N, keyof A>,
  fb: Tree<B>
) => (
  fa: Tree<A>
) => Tree<
  {
    readonly [K in N | keyof A]: K extends keyof A ? A[K] : B
  }
>
/**
 * @since 3.0.0
 */
export declare const ApT: Tree<readonly []>
/**
 * @since 3.0.0
 */
export declare const tupled: <A>(fa: Tree<A>) => Tree<readonly [A]>
/**
 * @since 3.0.0
 */
export declare const apT: <B>(
  fb: Tree<B>
) => <A extends ReadonlyArray<unknown>>(fas: Tree<A>) => Tree<readonly [...A, B]>
